#include <SwitecX25.h>
#include <Adafruit_NeoPixel.h>

 //information about your setup
#define STEPS (140*3) //sweep of speedometer in degrees multiplied by 3 to get number of steps
#define STEPStach (180*3) // sweep of tachometer in degrees multiplied by 3 to get number of steps
#define R1 22 //first resistor of voltage divider in kohms
#define R2 10 //second resistor of voltage divider in kohms
#define NUMPIXELS 8 //number of neopixels used as dash light, the voltmeter is additional to this
#define cylinders 8 
#define poles 2 //number of pickups on driveshaft revolution
#define maxmph 120.0
#define reargear 3.08
#define tirediameter  25.40
#define maxrpmtach 6000.0


 //pins used
#define PINNEOPIXEL 13
#define PINDASHLIGHTS 12
#define interupthall 3 //pin used for prox sensor for speedometer
#define interupttach 2 //pin used for isolation from ignition coil
#define analogInput 0 // for volt gauge
SwitecX25 motor2(STEPStach,4,5,6,7);//pins for tachometer stepper
SwitecX25 motor1(STEPS,8,9,10,11);//pins for speedometer steppe



byte DASHLIGHTSTATUS = 0;
byte DASHLIGHTSWITCH = 0;
byte voltmeterstatus = 0;
float sweep=STEPS;


//variables for tachometer/speedometer
volatile unsigned long lastmicros=0; //time counter for driveshaft
volatile unsigned long lastmicrostach=0; // time counter for 
volatile unsigned int mph= 1;
volatile unsigned int rpmtach = 1;
volatile unsigned long elapsed =0; // time between driveshaft readings
volatile unsigned long elaspedtach=0; //time btween ignition pulses
volatile  long zerocounter=0; //counter value between readings before setting steppers to zero
volatile long zerocountertach=0;
unsigned long setspeed;
unsigned long settach;
volatile int mphold=0; //compareable time value for old reading
volatile int rpmoldtach=0;
int filter=0; //rolling value for filter number of steps before requesting a new move
int filtertach=0;
byte driveshaftcounter=0;
byte enginecounter=0;

//variables for voltmeter
volatile unsigned int vin = 0;
volatile unsigned int value = 0;
const unsigned long  voltageMultiplier = ((R1+R2)*100/R2);
Adafruit_NeoPixel pixels = Adafruit_NeoPixel(60, PINNEOPIXEL, NEO_GRB + NEO_KHZ800);

void setup(){
// Serial.begin(9600); 
// Serial.println("SerialActive");
 //sweeps the steppers forward for homing.


 
 attachInterrupt(digitalPinToInterrupt(interupthall),pulsehall, FALLING);
 attachInterrupt(digitalPinToInterrupt(interupttach),pulsetach,FALLING);
  
  
 settach = (120000000/cylinders)/(maxrpmtach/STEPStach);//equation to find value to divide by time in microseconds to gain the number of steps
  
 setspeed = (3600000000/poles)/((63360.00*reargear)/(tirediameter*M_PI))*(sweep/maxmph); //equation to convert mph to steps


//Serial.println(voltageMultiplier);
// Serial.println(settach);
  //Serial.println(setspeed);
 pinMode(analogInput, INPUT);
 pinMode(PINDASHLIGHTS, INPUT);
 pixels.begin();
 pixels.show();
// mph = 1;
 motor1.zero();
 motor2.zero(); 

}

void loop(){
  if(enginecounter>0){ //increase this value from 0 to take an average of a few samples if the needle is jumpy
    elaspedtach = micros()-lastmicrostach;
    lastmicrostach = micros();
    rpmtach = (settach*enginecounter)/elaspedtach;
    enginecounter=0;
  }

  if(driveshaftcounter>0){ //increase this value from 0 to take an average of a few samples if the needle is jumpy
  elapsed = micros()-lastmicros;
     lastmicros = micros();
     mph  = (setspeed*driveshaftcounter)/elapsed;
   driveshaftcounter=0;
   //Serial.println(elapsed);

  }
 zerocounter = micros()-lastmicros;
 zerocountertach = micros()-lastmicrostach;
 //Serial.println(zerocountertach);


    
  //Serial.println(rpmtach);
 // Serial.println(mph);
  
   filter = mph-mphold;
   filter = abs(filter);
   
   filtertach = rpmtach-rpmoldtach; 
   filtertach = abs(filtertach);    
   
   if (filter > 6){;
    motor1.setPosition(mph);
    mphold = mph;
   //  Serial.print(mph);Serial.println(" MPH");
   }
    
    if (filtertach >6){
      motor2.setPosition(rpmtach);
      rpmoldtach = rpmtach;
    //Serial.println(rpmtach);  Serial.print(" RPM");
    }

  
 if (zerocounter >500000){   //will wait .5 seconds before setting stepper back to zero
      mph=0;
       motor1.setPosition(mph);
    }
  if (zerocountertach >250000){//will wait .25 seconds before setting stepper back to zero
     rpmtach=0;
     motor2.setPosition(rpmtach);
    // Serial.println(zerocountertach);
    }

//code for voltmeter to be read by neopixel color, off indicating car is running,green indicating strong battery and red indicating weak
   value = analogRead(analogInput);
   vin = (value*5*voltageMultiplier)/1024;
  //Serial.println(vin); 
   if (vin<90) {
   vin=0;
   } 
   if((vin>=1250)&&(voltmeterstatus==1)){             //12.5 volts is used here, remember the input diode has a voltage drop of about 0.7v
    pixels.setPixelColor(NUMPIXELS, pixels.Color(0,0,0)); 
    voltmeterstatus = 0;
    pixels.show();
   }
    if(vin<1250){
    pixels.setPixelColor(NUMPIXELS, pixels.Color((255-((vin-1000))),0,(vin-1000)) ); 
    voltmeterstatus = 1;
    pixels.show();}
 
  DASHLIGHTSWITCH= digitalRead(PINDASHLIGHTS); //code for dashlights on and off
   if((DASHLIGHTSWITCH == HIGH)&&(DASHLIGHTSTATUS==0)){
    dashLightsON();
   }
   if((DASHLIGHTSWITCH == LOW)&&(DASHLIGHTSTATUS==1)){
   dashLightsOFF();
   
  }
  //Serial.print(mph);Serial.println(" MPH");
 // Serial.println(rpmtach);
 motor1.update();
 motor2.update();
}








void pulsehall(){ //interupt for driveshaft
driveshaftcounter++;
 //Serial.println("driveshaft pulse");
}
void pulsetach(){// interupt for ignition pulses
enginecounter++;            
   // Serial.println(elaspedtach);
  }


void dashLightsON(){
 for(int i=0;i<(NUMPIXELS);i++){
  pixels.setPixelColor(i, pixels.Color(200,200,200));
  }
pixels.show();
DASHLIGHTSTATUS=1;
}


void dashLightsOFF(){
 for(int i=0;i<(NUMPIXELS);i++){
    pixels.setPixelColor(i, pixels.Color(0,0,0)); 
 }
DASHLIGHTSTATUS=0;
pixels.show();
}
